#[doc(hidden)]
#[macro_export]
macro_rules! generate_panic
{
   ($($arg: tt)+) =>
   ({
      use std::fmt;

      panic!(fmt::format(format_args!($($arg)+)));
   })
}

#[doc(hidden)]
#[macro_export]
macro_rules! log
{
   ($log_level: expr, $($arg: tt)+) =>
   ({
      // Specifically use the Path from std in this block.
      use std::path::Path;

      let file_path: &Path;
      let mut file_name: &str;

      file_name = file!();
      file_path = Path::new(file!());
      match file_path.file_name()
      {
         Some(name) =>
         {
            match name.to_str()
            {
               Some(string) =>
               {
                  file_name = string;
               }

               None =>
               {
               }
            }
         }

         None =>
         {
         }
      }

      println!("{0}: {1}::{2}::{3} -- {4}", $log_level,
               module_path!(), file_name, line!(),
               format_args!($($arg)+));
   })
}

#[macro_export]
macro_rules! error
{
   ($($arg: tt)*) =>
   ({
      use scribe::LogLevel;


      log!(LogLevel::ERROR, $($arg)*);
      generate_panic!($($arg)*);
   })
}

#[macro_export]
macro_rules! warn
{
   ($($arg: tt)*) =>
   ({
      use scribe::LogLevel;

      log!(LogLevel::WARN, $($arg)*);
   })
}

#[macro_export]
macro_rules! info
{
   ($($arg: tt)*) =>
   ({
      use scribe::LogLevel;

      log!(LogLevel::INFO, $($arg)*);
   })
}

#[macro_export]
macro_rules! debug
{
   ($($arg: tt)*) =>
   ({
      use scribe::LogLevel;

      // Only log this if we are currently in a debug build.
      if cfg!(debug_assertions) == true
      {
         log!(LogLevel::DEBUG, $($arg)*);
      }
   })
}
